/**
* @fileoverview Sample showing capturing a KML file click
*   and displaying the contents in a side panel instead of
*   an InfoWindow
*/

function get_cookie ( cookie_name ) {
  var cookie_string = document.cookie ;
  if (cookie_string.length != 0) {
    var cookie_value = cookie_string.match ( '(^|;)[\s]*' + cookie_name + '=([^;]*)' );
    return decodeURIComponent ( cookie_value[2] ) ;
  }

  return '' ;
}

var map = null;
var myParser = null;
var markerClusterer = null;

var src = './bin/kml/global.kml';

function showGlobalPins() {
    document.cookie = "pin_mode=global";
    
    var link = document.getElementById("globalPinLink");
    if (link) link.className = "activePinLink";
    link = document.getElementById("pagePinLink");
    if (link) link.className = "inactivePinLink";

    initialUpdate = true;
    loadKmlLayer("./bin/kml/global.kml", map);
}

function showPagePins() {
    document.cookie = "pin_mode=page";

    var kmlElement = document.getElementById("embeddedKML");
    var kml = "";
    
    if (kmlElement) {
        kml = kmlElement.innerText;
    }

    var link = document.getElementById("globalPinLink");
    if (link) link.className = "inactivePinLink";
    link = document.getElementById("pagePinLink");
    if (link) link.className = "activePinLink";

    initialUpdate = true;
    loadKmlString(kml, map);
}

function toDMS(pos) {
    var convertLat = Math.abs(pos);
    var LatDeg = Math.floor(convertLat);
    var LatMin = Math.floor((convertLat - LatDeg) * 60);
    var LatSec = Math.round((convertLat - (LatDeg + (LatMin/60))) * 3600);

    return LatDeg + "&deg;" + LatMin + "'" + LatSec + "\""
}

function makeClickHandler(pm) {
    return function(E) {
        var latCardinal = ((E.latLng.lat() > 0) ? "N" : "S");
        var lonCardinal = ((E.latLng.lng() > 0) ? "E" : "W");
        var latString = toDMS(E.latLng.lat());
        var lonString = toDMS(E.latLng.lng());
         
        infoBubble.setContent('<div class="gm-style"><h3>' + pm.marker.title + "</h3>"
            + '<div style="width: 100%; text-align: center;"><a href="' + pm.description + '"><img class="Thumbnail" src="' + pm.marker.icon.url + '"/></a></div>'
            + latString + latCardinal + ', ' + lonString + lonCardinal + '</div>'
            );
        infoBubble.open(map, pm.marker)
    };
}

function CM(placemark, doc) {
    doc.markers = doc.markers || [];

    var marker = null; 
 
    var icon = placemark.style.icon;
    if (!icon.marker && icon.img) {
        // yay, single point of failure is holding up multiple markers...
        icon.markerBacklog = icon.markerBacklog || [];
        icon.markerBacklog.push([placemark, doc]);
        return null;
    }

    // Load basic marker properties
    var markerOptions = geoXML3.combineOptions(geoXML3.parserOptions.markerOptions, {
    map: geoXML3.parserOptions.map,
        position: new google.maps.LatLng(placemark.Point.coordinates[0].lat, placemark.Point.coordinates[0].lng),
        title: placemark.name,
        zIndex: Math.round(placemark.Point.coordinates[0].lat * -100000) << 5,
        icon: icon.marker,
        shadow: icon.shadow,
        flat: !icon.shadow,
        visible: placemark.visibility
    });

    // Create the marker on the map
    var marker = new CustomMarker(markerOptions);
    if (!!doc) doc.markers.push(marker);

    placemark.marker = marker;

    if (marker) {
        markerClusterer.addMarker(marker);
        google.maps.event.addListener(marker, 'click', makeClickHandler(placemark));
    }

    return marker;
}

var doingLayout = false;
var initialUpdate = true;

function layoutCustomMarkers(doc) {
    if (markerClusterer == null || map == null || doingLayout) {
        return;
    }

    if (map.zoom <= markerClusterer.maxZoom) return;

    if (myParser) {
        if (myParser.docs[0]) {
            try {
                doingLayout = true;

                // Build an array containing all display rectangles, where they're actually displayed.
                var haveChanged = false;
                var haveProcessed = true;
                var bounds = map.getBounds();

                for (var pm = 0; pm < myParser.docs[0].markers.length; pm++) {
                    var mk = myParser.docs[0].markers[pm];

                    if (mk.map != null) {
                        mk.drawOffset = { x: 0, y: 0 };
                    }
                }

                // Run a few passes to try and get everything clear. Try and prevent the case where moved thumbs
                // then cover others.
                for (var pass = 0; pass < 3 && haveProcessed; pass++) {
                    haveProcessed = false;
                    var allRects = [];
                    
                    for (var pm = 0; pm < myParser.docs[0].markers.length; pm++) {
                        var mk = myParser.docs[0].markers[pm];

                        if (mk.map != null) {
                            var newRect = { marker: mk, rect: mk.getRectForZoom(map.zoom) };

                            newRect.rect.dl += Math.min(0, mk.drawOffset.x);
                            newRect.rect.dr += Math.max(0, mk.drawOffset.x);
                            newRect.rect.dt += Math.min(0, mk.drawOffset.y);
                            newRect.rect.db += Math.max(0, mk.drawOffset.y);

                            newRect.rect.r = newRect.rect.x + newRect.rect.w;
                            newRect.rect.b = newRect.rect.y + newRect.rect.h;
                            
                            allRects.push(newRect);
                        }
                    }

                    // Scan all displayed rectangles to see which overlap.
                    for (var pm = 0; pm < allRects.length; pm++) {
                        if (allRects[pm] != null) {
                            var collRects = [allRects[pm]];
                            var center = { x: allRects[pm].rect.x, y: allRects[pm].rect.y };

                            for (var coll = pm + 1; coll < allRects.length; coll++) {
                                if (allRects[coll] != null) {
                                    if (allRects[pm].rect.dl <= allRects[coll].rect.dr && allRects[pm].rect.dr >= allRects[coll].rect.dl &&
                                        allRects[pm].rect.dt <= allRects[coll].rect.db && allRects[pm].rect.db >= allRects[coll].rect.dt
                                    ) {
                                        // Overlap.
                                        collRects.push(allRects[coll]);
                                        center.x += allRects[coll].rect.x;
                                        center.y += allRects[coll].rect.y;

                                        allRects[coll] = null;      // Don't consider it again.
                                    }
                                }
                            }

                            if (collRects.length > 1) {
                                // Move each out in a circle from the centerpoint to a minimum distance.
                                var segAngle = 360.0 / collRects.length;
                                haveProcessed = true;
                                center.x /= collRects.length;
                                center.y /= collRects.length;

                                //                       alert("Rect " + pm + " has " + collRects.length + " colliders. Using " + segAngle);

                                for (var coll = 0; coll < collRects.length; coll++) {
                                    var localDeflect = Math.max(newRect.rect.w, newRect.rect.h);
                                    var deflection = 3 + 0.5 * localDeflect;
                                    if (collRects.length > 2) {
                                        deflection += Math.abs(localDeflect / Math.tan(segAngle * Math.PI / 180));
                                    }

                                    collRects[coll].marker.drawOffset.x = deflection * pass * Math.sin(segAngle * (coll + 0.5) * Math.PI / 180);
                                    collRects[coll].marker.drawOffset.y = deflection * pass * Math.cos(segAngle * (coll + 0.5) * Math.PI / 180);

                                    var markerPos = collRects[coll].marker.getProjection().fromDivPixelToLatLng(
                                        {
                                            x: collRects[coll].rect.r + Math.abs(collRects[coll].marker.drawOffset.x) + 1 * (collRects[coll].rect.w),
                                            y: collRects[coll].rect.b + Math.abs(collRects[coll].marker.drawOffset.y) + 1 * (collRects[coll].rect.h)
                                        }
                                    );

                                    if (!bounds.contains(markerPos)) {
                                        haveChanged = true;
                                        bounds.extend(markerPos);
                                    }

                                    markerPos = collRects[coll].marker.getProjection().fromDivPixelToLatLng(
                                        {
                                            x: (collRects[coll].rect.x - Math.abs(collRects[coll].marker.drawOffset.x)) - 1 * (collRects[coll].rect.w),
                                            y: (collRects[coll].rect.y - Math.abs(collRects[coll].marker.drawOffset.y)) - 1 * (collRects[coll].rect.h)
                                        }
                                    );
                                    if (!bounds.contains(markerPos)) {
                                        haveChanged = true;
                                        bounds.extend(markerPos);
                                    }

                                    collRects[coll].marker.draw();
                                }
                            }
                        }
                    }
                }

                if (haveProcessed) {
                    if (haveChanged && initialUpdate) {
                        google.maps.event.addListenerOnce(map, "idle", function() {
                            map.fitBounds(bounds);
                            map.panToBounds(bounds);
                        });
                    }

                    initialUpdate = false;
                }
            } catch (e) {
                
            }
        }
    }

    doingLayout = false;
}

/**
* Initializes the map
*/
function initialize() {
    map = new google.maps.Map(document.getElementById('map_canvas'), {
        center: new google.maps.LatLng(-19.257753, 146.823688),
        zoom: 2,
        mapTypeId: googleMapsMode
    });

    google.maps.event.addListener(map, 'zoom_changed', layoutCustomMarkers);

    if (useGlobalPins) {
        showPagePins();
    } else {
        showGlobalPins();
    }
}

function loadKmlString(src, map) {
    if (markerClusterer) {
        markerClusterer.clearMarkers();
        markerClusterer.setMap(null);
        markerClusterer = null;
    }
    
    if (myParser) {
        if (myParser.docs[0]) {
            myParser.hideDocument();
        }
    }

    markerClusterer = new MarkerClusterer(map, [], {
        gridSize: 40,
        maxZoom: 10
    });

//    google.maps.event.addListener(markerClusterer, 'clusteringend', layoutCustomMarkers);
    
    myParser = new geoXML3.parser({
        map: map,
        singleInfoWindow: true,
        createMarker: CM,
        processStyles: true,
        suppressInfoWindows: true
    });
    
    try {
        myParser.parseKmlString(src);
    } catch(e) {
    }
}
/**
* Adds a KMLLayer based on the URL passed. Clicking on a marker
* results in the balloon content being loaded into the right-hand div.
* @param {string} src A URL for a KML file.
*/
function loadKmlLayer(src, map) {
    if (markerClusterer) {
        markerClusterer.clearMarkers();
        markerClusterer.setMap(null);
        markerClusterer = null;
    }
    
    if (myParser) {
        if (myParser.docs[0]) {
            myParser.hideDocument();
        }
    }

    markerClusterer = new MarkerClusterer(map, [], {
        gridSize: 40,
        maxZoom: 10
    });
    
//    google.maps.event.addListener(markerClusterer, 'clusteringend', layoutCustomMarkers);
    
    myParser = new geoXML3.parser({
        map: map,
        singleInfoWindow: true,
        createMarker: CM,
        processStyles: true,
        suppressInfoWindows: true
    });
    
    try {
        myParser.parse(src);
    } catch(e) {
        if (document.location.protocol == 'file:') {
            alert("Many browsers are unable to load local files. Please try on a real webserver.");
        } else {
            alert("Unable to load the placemarks : " + e);
        }
    }
}

google.maps.event.addDomListener(window, 'load', initialize);
